from node import Node
import random

class Network:
    def __init__(self, args):
        self.args = args
        self.initialise_nodes()
        self.create_connections()

        pass

    def initialise_nodes(self):
        """
            Create a list of nodes with the specified parameters
        """
        self.nodes = []

        # Create a list of badwidth and cpu type based on z0 and z1
        # 0 is slow and 1 is fast
        bandwidth_type = [1] * self.args.num_nodes
        num_slow_bandwidth = int((self.args.z0*self.args.num_nodes)/100)
        rand_idx = random.sample(range(self.args.num_nodes), num_slow_bandwidth)
        for idx in rand_idx:
            bandwidth_type[idx] = 0

        cpu_type = [1] * self.args.num_nodes
        num_slow_cpu = int((self.args.z1*self.args.num_nodes)/100)
        rand_idx = random.sample(range(self.args.num_nodes), num_slow_cpu)
        for idx in rand_idx:
            cpu_type[idx] = 0

        # Based on CPU type calculate the PoW time Tk
        # 10 * h_k * (n - num_slow) + h_k * num_slow = 1
        # => h_k = 1 / (10 * n - 9 * num_slow)
            
        h_k_slow = 1 / float(10*self.args.num_nodes - 9*num_slow_cpu)
        Tk = []
        for i in range(self.args.num_nodes):
            if cpu_type[i] == 0:
                Tk.append(h_k_slow)
            else:
                Tk.append(10 * (1 - h_k_slow))

        for i in range(self.args.num_nodes):
            self.nodes.append(Node(i, bandwidth_type[i], cpu_type[i], Tk[i]))
        return
    
    def create_connections(self):
        """
            Create a bidirectional graph, 3-6 adjacent nodes per node
            TODO(SM): Need a better way to create a valid graph
        """
        num_nodes = self.args.num_nodes
        valid_graph = False
        cnt = 0
        while valid_graph == False:
            print("tyring to create a valid graph",cnt)
            adjacency_list = [[] for _ in range(num_nodes)]
            for i in range(num_nodes):
                num_connections = random.randint(3, 6)
                potential_neighbors = [x for x in range(num_nodes) if x != i and len(adjacency_list[x]) < 6]
                neighbors = random.sample(potential_neighbors, min(num_connections, len(potential_neighbors)))
                for j in neighbors:
                    adjacency_list[i].append(j)
                    adjacency_list[j].append(i)

            valid_graph = True
            for i in range(num_nodes):
                if len(adjacency_list[i]) < 3 or len(adjacency_list[i]) > 6:
                    valid_graph = False
                    break
            #print(adjacency_list)
            cnt += 1
        for i in range(num_nodes):
            for j in adjacency_list[i]:
                self.nodes[i].neighbors.append(id(self.nodes[j]))
        print("Graph created")
        return

    def calculate_latency(sender, receiver, message_size):
        # Calculate latency based on provided formula
        pass
    
    def forward_transaction(sender, receiver, transaction):
        # Forward transaction from sender to receiver if conditions are met
        pass