from network import Network
import time
import random

class Event:
    def __init__(self, time, operation, args):
        self.time = time
        self.operation = operation
        self.args = args
    def process(self):
        print("Processing event at time", self.time)
        return self.operation(self.args)

class EventSimulator:
    """
        Simulate the network
        call startSimulation to start the simulation
    """
    def __init__(self, args):
        self.nx = Network(args)
        self.queue = []
        self.sim_time = args.sim_time
        self.curr_time = 0
        self.pij = random.randrange(10, 500) # speed of light propagation delay in ms
        
    def startSimulation(self):
        """
            Start the simulation. 
            Run till the simulation time is reached or the queue is empty
        """
        while len(self.queue) > 0 and self.queue[0].time < self.sim_time:
            event = self.queue.pop(0)
            new_events = event.process()
            self.push_new_events(new_events)
        return

    def push_new_events(self, events):
        """
            Add each event to the queue just before the time it is supposed to occur
        """
        for event in events:
            for i in range(len(self.queue)):
                if self.queue[i].time > event.time:
                    self.queue.insert(i, event)
                    break
            else:
                self.queue.append(event)
